#ifndef __XMLIO_CALENDAR__
#define __XMLIO_CALENDAR__

namespace XMLIOSERVER
{
   class JulianCalendar : public AbstractCalendar
   {
      public :

         JulianCalendar(const string& _dateStr)
            : AbstractCalendar("Julian", _dateStr)
         { /* Ne rien faire de plus */ }

         JulianCalendar(int yr = 0, int mth = 1, int d   = 1,
                        int hr = 0, int min = 0, int sec = 0)
             : AbstractCalendar("Julian", yr, mth, d, hr, min, sec)
         { /* Ne rien faire de plus */ }

      public : /* virtual */

         virtual int getYearTotalLength(const Date& _d) const
         { // Retourne la durée d'une année en seconde.
            if (_d.getYear()%4 == 0)
               return (366 * 86400);
            return (365 * 86400);
         }

         virtual int getMonthLength(const Date& _d) const
         { // Retourne la durée du mois en jour.
            if (_d.getMonth() == 2)
            {
               if (_d.getYear()%4 == 0)
                  return 29;
               return 28;
            }
            return (AbstractCalendar::getMonthLength(_d));
         }

         virtual string getType(void) const
         { return (string("julian")); }

         virtual ~JulianCalendar(void)
         { /* Ne rien faire de plus */ }

   }; // class JulianCalendar

   class GregorianCalendar : public AbstractCalendar
   {
      public :

         GregorianCalendar(int yr = 0, int mth = 1, int d   = 1,
                           int hr = 0, int min = 0, int sec = 0)
             : AbstractCalendar("Gregorian", yr, mth, d, hr, min, sec)
         { /* Ne rien faire de plus */ }

         GregorianCalendar(const string& dateStr)
            : AbstractCalendar("Gregorian", dateStr)
         { /* Ne rien faire de plus */ }

      public : /* virtual */

         virtual int getYearTotalLength(const Date& d) const
         { // Retourne la durée d'une année en seconde.
            if ((d.getYear() % 4   == 0) &&
               ((d.getYear() % 100 != 0)  ||
                (d.getYear() % 400 == 0)  ))
               return (366 * 86400);
            return (365 * 86400);
         }

         virtual int getMonthLength(const Date& d) const
         { // Retourne la durée du mois en jour.
            if (d.getMonth() == 2)
            { // Traitement du cas particulier en Février.
               if ((d.getYear() % 4   == 0) &&
                  ((d.getYear() % 100 != 0) ||
                   (d.getYear() % 400 == 0) ))
                  return (29);
               return (28);
            }
            return (AbstractCalendar::getMonthLength(d));
         }

         virtual string getType(void) const
         { return (string("gregorian")); }

         virtual ~GregorianCalendar(void)
         { /* Ne rien faire de plus */ }

   }; // class GregorianCalendar

   class NoLeapCalendar : public AbstractCalendar
   {
      public :

         NoLeapCalendar(int yr = 0, int mth = 1, int d   = 1,
                        int hr = 0, int min = 0, int sec = 0)
             : AbstractCalendar("NoLeap", yr, mth, d, hr, min, sec)
         { /* Ne rien faire de plus */ }

         NoLeapCalendar(const string& dateStr)
            : AbstractCalendar("NoLeap", dateStr)
         { /* Ne rien faire de plus */ }

      public : /* virtual */

         virtual string getType(void) const
         { return (string("noleap")); }

         virtual ~NoLeapCalendar(void)
         { /* Ne rien faire de plus */ }

   }; // class NoLeapCalendar

   class AllLeapCalendar : public AbstractCalendar
   {
      public :

         AllLeapCalendar(int yr = 0, int mth = 1, int d   = 1,
                         int hr = 0, int min = 0, int sec = 0)
             : AbstractCalendar("AllLeap", yr, mth, d, hr, min, sec)
         { /* Ne rien faire de plus */ }

         AllLeapCalendar(const string& dateStr)
            : AbstractCalendar("AllLeap", dateStr)
         { /* Ne rien faire de plus */ }

      public : /* virtual */

         virtual int getMonthLength(const Date& d) const
         {
            if (d.getMonth() == 2)
               return (29);
            return (AbstractCalendar::getMonthLength(d));
         }

         virtual int getYearTotalLength(const Date& d) const
         { return (366 * 86400); }

         virtual string getType(void) const
         { return (string("allleap")); }

         virtual ~AllLeapCalendar(void)
         { /* Ne rien faire de plus */ }

   }; // class NoLeapCalendar

   class D360Calendar : public AbstractCalendar
   {
      public :

         D360Calendar(int yr = 0, int mth = 1, int d   = 1,
                      int hr = 0, int min = 0, int sec = 0)
             : AbstractCalendar("D360", yr, mth, d, hr, min, sec)
         { /* Ne rien faire de plus */ }

         D360Calendar(const string& dateStr)
            : AbstractCalendar("D360", dateStr)
         { /* Ne rien faire de plus */ }

      public : /* virtual */

         virtual int getYearTotalLength(const Date& d) const
         { return (360 * 86400); }

         virtual int getMonthLength(const Date& d) const
         { return (30); }

         virtual string getType(void) const
         { return (string("360d")); }

         virtual ~D360Calendar(void)
         { /* Ne rien faire de plus */ }

   }; // class D360Calendar

} // namespace XMLIOSERVER

#endif // __XMLIO_CALENDAR__
